/* APPLCMDS.C -- LONWORKS(R) host application command handler.
 * Copyright (c) 1994 by Echelon Corporation.
 * All Rights Reserved.
 *
 * Date last modified: 11/17/94
 *
 * Handle user commands to the host application
 */

#include "ni_msg.h"
#include "ni_mgmt.h"
#include "hauif.h"
#include "applmsg.h"
#include <stdio.h>
#include <string.h>
#include <conio.h>
#include <stdlib.h>
#include <ctype.h>

/*************************************************************
 *
 *          askNvIndex - prompt user to enter NV index
 *
 *************************************************************/

static int askNvIndex( Bool inOnly ) {   // returns -1 if invalid

    static const char * dirString[ 2 ] = { "IN ", "OUT" };
    int         nvIndex;
    NvData   * pNvValue;
    nv_struct * pNvConfig;

    pNvValue = NvValueTable;
    pNvConfig = NvConfigTable;
        // list all the network variables - menu label is index+1

    for( nvIndex = 0; nvIndex < NUM_NVS;
         nvIndex++, pNvValue++, pNvConfig++ ) {
        if( inOnly && pNvConfig->direction != NV_IN ) continue;
                             // don't list if not asked for
        printf( "%d. \t%s\t%s\t", nvIndex + 1,
            dirString[ pNvConfig->direction ], pNvValue->name );
        pNvValue->printFunc( pNvValue->data );       // print value
    }

    nvIndex = getInteger( "Enter NV number :", 0 ) - 1;
    // If user enters 0 (the default), return error

    if( nvIndex >= NUM_NVS || nvIndex < 0 ) {
        printf( "Invalid NV number\n" );
        return -1;
    }

    pNvConfig = & NvConfigTable[ nvIndex ];

    if( inOnly && pNvConfig->direction != NV_IN ) {
        printf( "Invalid direction\n" );
        return -1;
    }
    return nvIndex;
}

/*************************************************************
 *
 *          handleNetvarPollResponse
 *
 *************************************************************/

static void handleNetvarPollResponse(
       UnprocessedNV * inNv,
       Byte            inLength,
       Word            nvIndex ) {

    nv_struct         * pNvConfig;

    if( reportFlag )printf( "Received network variable poll response\n" );
    // Match on direction and selector bits ( not the first bit )

    pNvConfig = & NvConfigTable[ nvIndex ];   // point to NV config table

    if( inNv->NV_selector_lo != pNvConfig->selector_lo ) return;
    if( inNv->NV_selector_hi != pNvConfig->selector_hi ) return;
    if( inNv->direction != pNvConfig->direction ) return;

    ( void )updateNvValue( nvIndex, inNv->data, inLength - 2 );
        // data size is msg size less selector size
}

/*************************************************************
 *
 *          Command to poll an input network variable
 *
 *************************************************************/

void NV_poll( void ) {

    int           nvIndex, taNvIndex;
    NICode        niError;
    ComplType     completion;
    int           numResponses;
    Byte          inLength;
    MsgData       inData, outData;
    nv_struct   * pNvConfig;
    NvData      * pNvValue;

    printf( "Which input network variable do you wish to poll?\n" );
    nvIndex = askNvIndex ( TRUE );      // get index of NV from user
    if( nvIndex < 0 || ! onlineFlag ) return;
                // all done if NV not found, or node is off-line
    pNvConfig = & NvConfigTable[ nvIndex ];   // point to cnfg table

    // make an outgoing request message to poll the NV
    outData.unv.direction = NV_OUT;        // to match the receiver(s)
    outData.unv.NV_selector_hi = pNvConfig->selector_hi;
    outData.unv.NV_selector_lo = pNvConfig->selector_lo;
    outData.unv.must_be_one = 1;

    if( pNvConfig->addr_index != NULL_IDX ) {     // NV is bound

            // For implicit addressing, use configured address table entry

        niError = NiSendMsgWait(
            REQUEST,
            NULL,                   // implicit destination address
            & outData,
            2,                      // size of selector
            pNvConfig->priority,
            pNvConfig->auth,
            & completion,
            & numResponses,
            NULL,                  // response address
            & inData,              // first response data
            & inLength,            // input length
	    pNvConfig->addr_index );	 // address table index

        if( NiHandleError( niError, completion, NO_CHECK,
                "polling NV" ) ) return;

        if( numResponses ) for( ;; ) {       // handle responses, if any

            handleNetvarPollResponse( & inData.unv, inLength,
                nvIndex );  // update the value if a valid response

            if( ! --numResponses ) break;

            niError = NiGetNextResponse(    // get another, if a group
                NULL,                       // response address
                & inData,                   // response data
                & inLength );

            if( NiHandleError( niError, completion, NO_CHECK,
                "polling NV" ) ) return;
        }
    }
    if( !pNvConfig->turnaround ) return;
            // all done if not a turnaround

    // get the value of the matching output
    taNvIndex = matchNvConfig( & outData.unv );
    if( taNvIndex < 0 ) return;       // no match

    pNvValue = & NvValueTable[ taNvIndex ]; // point to NV value table
    ( void )updateNvValue( nvIndex, pNvValue->data, pNvValue->size );
}

/*************************************************************
 *
 *          Command to update a network variable
 *
 *************************************************************/

void NV_update( void ) {

    NvData       * pNvValue;
    nv_struct    * pNvConfig;
    int            nvIndex;
    Byte           outLength;
    NICode         niError;
    ComplType      completion;
    MsgData        outData;

    printf( "Which network variable do you wish to update?\n" );
    nvIndex = askNvIndex ( FALSE );          // get NV index from user
    if( nvIndex < 0 ) return;

    pNvValue = & NvValueTable[ nvIndex ];    // point to value table
    pNvConfig = & NvConfigTable[ nvIndex ];    // point to config table

    printf( "Enter data for %s :", pNvValue->name );
    pNvValue->readFunc( pNvValue->data );    // get NV data from user

    if( pNvConfig->direction == NV_IN || ! onlineFlag ) return;
            // all done if an input or node is not on-line

    // make an outgoing NV update message

    outData.unv.direction = NV_IN;   // to match the receiver
    outData.unv.NV_selector_hi = pNvConfig->selector_hi;
    outData.unv.NV_selector_lo = pNvConfig->selector_lo;
    outData.unv.must_be_one = 1;

    outLength = pNvValue->size;
    memcpy( outData.unv.data, pNvValue->data, outLength );

    if( pNvConfig->addr_index != NULL_IDX ) { // NV is bound

        // For implicit addressing, use configured address table entry

        niError = NiSendMsgWait(
	    pNvConfig->service,    // use configured service type
            NULL,                  // implicit addressing
            & outData,
	    (Byte)(outLength + 2), // including selector
            pNvConfig->priority,
            pNvConfig->auth,
            & completion,
            NULL,                   // numResponses
            NULL,                   // response address
            NULL,                   // response data
            NULL,                   // input length
            pNvConfig->addr_index );
        ( void )NiHandleError( niError, completion, NO_CHECK,
                "updating NV" );
    }

    if( ! pNvConfig->turnaround ) return; // all done if not a turnaround
     // find the matching input NV
    ( void ) handleNetvarUpdate( & outData.unv, outLength, TRUE );
}

/*
 ****************************************************************************
 *
 *      NV_table() - command to display node's NV config table
 *
 ****************************************************************************
 */

void NV_table( void ) {

    nv_struct   * pNvConfig;
    NvData      * pNvValue;
    int           nvIndex;

    printf( "Index\tSelctr\tDir\tPrio\tAuth\tAddridx\tService\tTrnArnd\n" );

    pNvConfig = NvConfigTable;
    pNvValue = NvValueTable;

    for( nvIndex = 0; nvIndex < NUM_NVS; nvIndex++ ) {

        printf( "%d\t", nvIndex );
        printf( "%2.2x%2.2x\t", pNvConfig->selector_hi,
            pNvConfig->selector_lo );
        printf( pNvConfig->direction ? "out\t" : "in\t" );
        printf( pNvConfig->priority ? "yes\t" : "no\t" );
        printf( pNvConfig->auth ? "yes\t" : "no\t" );
        printf( "%d\t", pNvConfig->addr_index );
        printf( "%s\t", svcTable[ pNvConfig->service ] );
        printf( pNvConfig->turnaround ? "yes  " : "no   " );
        printf( "%s\n", pNvValue->name );

        pNvConfig++;       // go to next
        pNvValue++;
    }
}

/*************************************************************
 *
 *         User interface routines for SNVTs
 *
 *************************************************************/

void printAsc( Byte * data ) {     // print a value of type SNVT_str_asc
    printf( "\"%s\"\n", data );
}

/*************************************************************/

void printCont( Byte * data ) {    // print a value of type SNVT_lev_cont
    printf( "%g percent\n", * data / 2.0 );
}

/*************************************************************/

void printDisc( Byte * data ) {    // print a value of type SNVT_lev_disc

    switch ( * data ) {
        case 0:  printf( "OFF\n" );    return;
        case 1:  printf( "LOW\n" );    return;
        case 2:  printf( "MEDIUM\n" ); return;
        case 3:  printf( "HIGH\n" );   return;
        case 4:  printf( "ON\n" );     return;
        default: printf( "Invalid discrete value %d\n", * data );
    }
}

/*************************************************************/

void printFloat( Byte * data ) {   // print a value of type SNVT_xxx_f
    union {
        float   intelFloat;
        Byte    neuronFloat[ 4 ];
    } swapper;
    int   i;

    for( i = 3; i >= 0; i-- )
        swapper.neuronFloat[ i ] = * data++;       // reverse byte order
    printf( "%.5G\n", swapper.intelFloat );
}

/*************************************************************/

void readAsc( Byte * data ) {      // read a value of type SNVT_str_asc
    char inputBuf[ 80 ];

    gets( inputBuf );
    memset( data, 0, 31 );                      // null it out
    strncpy( ( char * )data, inputBuf, 31 );    // copy up to 31 bytes
}

/*************************************************************/

void readDisc( Byte * data ) {    // read a value of type SNVT_lev_disc
    char ch;
    for( ;; ) {
        printf( "\nO(F)F, (L)OW, (M)EDIUM, (H)IGH or O(N) :" );
        ch = getche( );
        switch( toupper( ch ) ) {
            case 'F':  * data = 0; break;
            case 'L':  * data = 1; break;
            case 'M':  * data = 2; break;
            case 'H':  * data = 3; break;
            case 'N':  * data = 4; break;
            default : printf( " -- invalid choice" ); continue;
        }
        printf( "\n" );
        return;
    }
}

/*************************************************************/

void readCont( Byte * data ) {         // read a value of type SNVT_lev_cont
    * data = 2 * getInteger( "(percent) ", * data / 2 );
}

/*************************************************************/

void readFloat( Byte * data ) {   // read a value of type SNVT_xxx_f
    union {
        float   intelFloat;
        Byte    neuronFloat[ 4 ];
    } swapper;
    int   i;
    char  inputBuf[ 80 ];

    for( ;; ) {
        gets( inputBuf );
        if( sscanf( inputBuf, "%g", & swapper.intelFloat ) == 1 ) break;
        printf( "Invalid floating point data\n" );
    }

    for( i = 3; i >= 0; i-- )
        * data ++ = swapper.neuronFloat[ i ];       // reverse byte order
}
