/* APPLMSG.C -- LONWORKS(R) host application message handler.
 * Copyright (c) 1993,1994 by Echelon Corporation.
 * All Rights Reserved.
 *
 * Date last modified: 11/17/94
 *
 * Handle application messages, network variable messages and network
 * management messages addressed to the application.
 */

#include "ni_msg.h"
#include "ni_mgmt.h"
#include "hauif.h"
#include "applmsg.h"
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <io.h>
#include <stdlib.h>

/*************************************************************
 *
 *          Declare network variable configuration table
 *
 *************************************************************/

nv_struct NvConfigTable[ NUM_NVS ]  // configuration table
    = {
/*        selhi  dir     prio  sello  addridx   auth  svc   trnarnd    */
        { 0x3F, NV_OUT, FALSE, 0xFF, NULL_IDX, FALSE, ACKD, FALSE },
        { 0x3F, NV_OUT, FALSE, 0xFE, NULL_IDX, FALSE, ACKD, FALSE },
        { 0x3F, NV_OUT, FALSE, 0xFD, NULL_IDX, FALSE, ACKD, FALSE },
        { 0x3F, NV_OUT, FALSE, 0xFC, NULL_IDX, FALSE, ACKD, FALSE },
        { 0x3F, NV_IN , FALSE, 0xFB, NULL_IDX, FALSE, ACKD, FALSE },
        { 0x3F, NV_IN , FALSE, 0xFA, NULL_IDX, FALSE, ACKD, FALSE },
        { 0x3F, NV_IN , FALSE, 0xF9, NULL_IDX, FALSE, ACKD, FALSE },
        { 0x3F, NV_IN , FALSE, 0xF8, NULL_IDX, FALSE, ACKD, FALSE }
    };                                     // saved to disk


/*************************************************************
 *
 *          Declare SNVT self identification storage
 *
 *************************************************************/

/* There are eight Network Variables and three message tags defined
   These are the equivalent Neuron C declarations:

    #pragma enable_sd_nv_names
    #pragma set_node_sd_string "Sample host application program"
    #pragma set_id_string "HostAppl"

    network output sd_string( "ASCII string output NV" )
            SNVT_str_asc  NV_string_out;
    network output sd_string( "Discrete output NV" )
            SNVT_lev_disc NV_disc_out;
    network output sd_string( "Continuous output NV" )
            SNVT_lev_cont NV_cont_out;
    network output sd_string( "Floating count output NV" )
            SNVT_count_f  NV_float_out;

    network input sd_string( "ASCII string input NV" )
            SNVT_str_asc  NV_string_in;
    network input sd_string( "Discrete input NV" )
            SNVT_lev_disc NV_disc_in;
    network input sd_string( "Continuous input NV" )
            SNVT_lev_cont NV_cont_in;
    network input sd_string( "Floating count input NV" )
            SNVT_count_f  NV_float_in;

    msg_tag tag_1;
    msg_tag tag_2;
    msg_tag tag_3;

*/

// Network variable names

#define SO_name    "NV_string_out"
#define DO_name    "NV_disc_out"
#define CO_name    "NV_cont_out"
#define FO_name    "NV_float_out"
#define SI_name    "NV_string_in"
#define DI_name    "NV_disc_in"
#define CI_name    "NV_cont_in"
#define FI_name    "NV_float_in"

// Self-documentation strings

#define SO_doc    "ASCII string output NV"
#define DO_doc    "Discrete output NV"
#define CO_doc    "Continuous output NV"
#define FO_doc    "Floating count output NV"
#define SI_doc    "ASCII string input NV"
#define DI_doc    "Discrete input NV"
#define CI_doc    "Continuous input NV"
#define FI_doc    "Floating count input NV"

#define Node_doc  "Sample host application program"

const struct {
    snvt_struct_v0      header;            // Version 0 SNVT_struct
    snvt_desc_struct    stringOutDesc;
    snvt_desc_struct    discOutDesc;
    snvt_desc_struct    contOutDesc;
    snvt_desc_struct    floatOutDesc;
    snvt_desc_struct    stringInDesc;
    snvt_desc_struct    discInDesc;
    snvt_desc_struct    contInDesc;
    snvt_desc_struct    floatInDesc;

    char                nodeDocString[ sizeof( Node_doc ) ];

    snvt_ext_rec_mask   stringOutMask;
    char                stringOutName[ sizeof( SO_name ) ];
    char                stringOutDoc[  sizeof( SO_doc ) ];
    snvt_ext_rec_mask   discOutMask;
    char                discOutName[   sizeof( DO_name ) ];
    char                discOutDoc[    sizeof( DO_doc ) ];
    snvt_ext_rec_mask   contOutMask;
    char                contOutName[   sizeof( CO_name ) ];
    char                contOutDoc[    sizeof( CO_doc ) ];
    snvt_ext_rec_mask   floatOutMask;
    char                floatOutName[  sizeof( FO_name ) ];
    char                floatOutDoc[   sizeof( FO_doc ) ];
    snvt_ext_rec_mask   stringInMask;
    char                stringInName[  sizeof( SI_name ) ];
    char                stringInDoc[   sizeof( SI_doc ) ];
    snvt_ext_rec_mask   discInMask;
    char                discInName[    sizeof( DI_name ) ];
    char                discInDoc[     sizeof( DI_doc ) ];
    snvt_ext_rec_mask   contInMask;
    char                contInName[    sizeof( CI_name ) ];
    char                contInDoc[     sizeof( CI_doc ) ];
    snvt_ext_rec_mask   floatInMask;
    char                floatInName[   sizeof( FI_name ) ];
    char                floatInDoc[    sizeof( FI_doc ) ];

} SnvtInfo = {
                                                  // SNVT structure
    { sizeof( SnvtInfo ) >> 8,    // length_hi
      sizeof( SnvtInfo ) & 0xFF,  // length_lo
      NUM_NVS,                    // num_netvars
      0,                          // version
      3                           // mtag_count
    },

    // These SNVT descriptors have ext_rec, nv_service_type_config,
    // nv_priority_config and nv_auth_config set.  In addition, the inputs
    // have nv_polled set, since the application polls them.

    { 0, 1, 1, 1, 0, 0, 0, 1, SNVT_str_asc },    // SNVT descriptors
    { 0, 1, 1, 1, 0, 0, 0, 1, SNVT_lev_disc },
    { 0, 1, 1, 1, 0, 0, 0, 1, SNVT_lev_cont },
    { 0, 1, 1, 1, 0, 0, 0, 1, SNVT_count_f  },
    { 0, 1, 1, 1, 0, 1, 0, 1, SNVT_str_asc  },
    { 0, 1, 1, 1, 0, 1, 0, 1, SNVT_lev_disc },
    { 0, 1, 1, 1, 0, 1, 0, 1, SNVT_lev_cont },
    { 0, 1, 1, 1, 0, 1, 0, 1, SNVT_count_f  },

    Node_doc,                       // node self-documentation string

    { 0, 0, 1, 1, 0, 0 },           // extension record mask
    SO_name,                        // name
    SO_doc,                         // self-doc

    { 0, 0, 1, 1, 0, 0 },           // extension record mask
    DO_name,                        // name
    DO_doc,                         // self-doc

    { 0, 0, 1, 1, 0, 0 },           // extension record mask
    CO_name,                        // name
    CO_doc,                         // self-doc

    { 0, 0, 1, 1, 0, 0 },           // extension record mask
    FO_name,                        // name
    FO_doc,                         // self-doc

    { 0, 0, 1, 1, 0, 0 },           // extension record mask
    SI_name,                        // name
    SI_doc,                         // self-doc

    { 0, 0, 1, 1, 0, 0 },           // extension record mask
    DI_name,                        // name
    DI_doc,                         // self-doc

    { 0, 0, 1, 1, 0, 0 },           // extension record mask
    CI_name,                        // name
    CI_doc,                         // self-doc

    { 0, 0, 1, 1, 0, 0 },           // extension record mask
    FI_name,                        // name
    FI_doc                          // self-doc
};

/*************************************************************
 *
 *          Declare network variable storage
 *
 *************************************************************/

NvData NvValueTable[ NUM_NVS ] =  {     // RAM storage for NVs
     { 31, SnvtInfo.stringOutName, printAsc,   readAsc   },
     {  1, SnvtInfo.discOutName,   printDisc,  readDisc  },
     {  1, SnvtInfo.contOutName,   printCont,  readCont  },
     {  4, SnvtInfo.floatOutName,  printFloat, readFloat },
     { 31, SnvtInfo.stringInName,  printAsc,   readAsc   },
     {  1, SnvtInfo.discInName,    printDisc,  readDisc  },
     {  1, SnvtInfo.contInName,    printCont,  readCont  },
     {  4, SnvtInfo.floatInName,   printFloat, readFloat }
};

        // Statistics accumulators for received traffic

unsigned numMessages       = 0;
unsigned numPolls          = 0;
unsigned numUpdates        = 0;
unsigned long dataLength   = 0;

Bool reportFlag  = TRUE;     // report incoming application msgs
Bool verboseFlag = FALSE;    // display all network interface msgs
Bool onlineFlag  = TRUE;     // this node is on-line

/*************************************************************
 *
 *          Prototypes for internal functions
 *
 *************************************************************/

Word getNvIndex( ExpAppBuffer * pMsgIn, nv_struct ** pNextByte );
    // get the nv index from nv_cnfg and nv_fetch messages


#define IN_MSG_HDR  pMsgIn->msg_hdr.exp      /* shorthand */

/*************************************************************
 *
 *          Handle incoming message to update NV config table
 *
 *************************************************************/

Bool NiHandleUpdateNvConfig( ExpAppBuffer * pMsgIn ) {

    nv_struct     * pNvConfig;
    Word            nvIndex;
    MsgData         response;
    NICode          niError;
    Bool            request;

    // When an update_nv_cnfg message is received, the new NV config
    // is copied to the NV config table of this node

    nvIndex = getNvIndex( pMsgIn, & pNvConfig );
                                // get requested index from msg

    if( nvIndex >= NUM_NVS )    // too big an index
        response.exp.code = NM_update_nv_cnfg_fail;
    else {
        response.exp.code = NM_update_nv_cnfg_succ;
        NvConfigTable[ nvIndex ] = * pNvConfig;
            // save nv config table entry
    }
    request = ( IN_MSG_HDR.st == REQUEST );    // is it a request msg?

    if( request ) niError = NiSendResponse( & response, 1, IN_MSG_HDR.tag );

    if( reportFlag ) printf( "Received Update NV Config network mgmt msg "
        "for index %d\n", nvIndex );

    if( request ) NiErrorDisplay( "Error sending response to Update NV Config",
        niError );
    return reportFlag || verboseFlag || ( niError != NI_OK );
}

/*************************************************************
 *
 *          Handle incoming message to query NV config table
 *
 *************************************************************/

Bool NiHandleQueryNvConfig( ExpAppBuffer * pMsgIn ) {

    Word     nvIndex;
    NM_query_nv_cnfg_response   nvConfigRspMsg;
    Byte     outLength;
    NICode   niError;

    // When a query_nv_cnfg message is received, the corresponding
    // entry in the NV config table of this node is returned in a
    // response

    if( IN_MSG_HDR.st != REQUEST ) return FALSE;
    nvIndex = getNvIndex( pMsgIn, NULL );
                           // get requested nvIndex from message

    if( nvIndex >= NUM_NVS ) {   // too big an index
        nvConfigRspMsg.code = NM_query_nv_cnfg_fail;
        outLength = 1;
    } else {
        nvConfigRspMsg.code = NM_query_nv_cnfg_succ;
        nvConfigRspMsg.nv_cnfg = NvConfigTable[ nvIndex ];
                   // get nv config table entry
        outLength = sizeof( NM_query_nv_cnfg_response );
    }
    niError = NiSendResponse( ( MsgData * ) & nvConfigRspMsg, outLength,
        IN_MSG_HDR.tag );

    if( reportFlag ) printf( "Received Query NV Config network mgmt msg "
        "for index %u\n", nvIndex );

    NiErrorDisplay( "Error sending response to Query NV Config", niError );
    return reportFlag || verboseFlag || ( niError != NI_OK );
}

/*************************************************************
 *
 *      Handle incoming message to set the mode of this node
 *
 *************************************************************/

Bool NiHandleSetNodeMode( ExpAppBuffer * pMsgIn ) {
    NICode    niError;

    // When a set_node_mode (on/off) line is received, the corresponding
    // command is sent down to the network interface

    onlineFlag = ( pMsgIn->data.exp.data[ 0 ] == APPL_ONLINE );
    niError = NiSendImmediate( onlineFlag ? niONLINE : niOFFLINE );

    if( reportFlag ) printf( "Received Set Node Mode network mgmt msg: "
        "set host node to %sline\n", onlineFlag ? "on" : "off" );

    NiErrorDisplay( "Error sending downlink mode change", niError );
    return reportFlag || verboseFlag || ( niError != NI_OK );
}

/*************************************************************
 *
 *      Handle incoming message to query this node's SNVT info
 *
 *************************************************************/

Bool NiHandleQuerySnvt( ExpAppBuffer * pMsgIn ) {

    Byte        outLength;
    MsgData     response;
    Word        offset;
    NICode      niError;
    NM_query_SNVT_request * querySnvtReqMsg;

    if( IN_MSG_HDR.st != REQUEST ) return FALSE;
    querySnvtReqMsg = ( NM_query_SNVT_request * ) & pMsgIn->data.exp;

        // get the offset as a 16-bit unsigned number

    offset = makeWord( querySnvtReqMsg->offset_hi,
                       querySnvtReqMsg->offset_lo );
    outLength = querySnvtReqMsg->count; // number of bytes requested

    if( offset + outLength > sizeof( SnvtInfo ) ) {
         response.exp.code = NM_query_SNVT_fail;
         outLength = 1;            // invalid request
    } else {
        response.exp.code = NM_query_SNVT_succ;
        memcpy( response.exp.data, ( Byte * ) & SnvtInfo + offset,
            outLength );   // copy data from the SNVT table to the response
        outLength++;       // for the code
    }

    niError = NiSendResponse( & response, outLength, IN_MSG_HDR.tag );

    if( reportFlag )printf( "Received Query SNVT network mgmt msg "
        "for offset %u\n", offset );

    NiErrorDisplay( "Error sending response to Query SNVT", niError );
    return reportFlag || verboseFlag || ( niError != NI_OK );
}

/*************************************************************
 *
 *      Handle incoming NM message to fetch the value of an NV
 *
 *************************************************************/

Bool NiHandleNvFetch( ExpAppBuffer * pMsgIn ) {

    Word         nvIndex;
    MsgData      response;
    Byte         outLength;
    NvData     * pNvValue;
    Byte       * pResponse;
    const char * nvName;
    NICode       niError;

    if( IN_MSG_HDR.st != REQUEST ) return FALSE;

     // When an NV_fetch message is received, value is fetched
    nvIndex = getNvIndex( pMsgIn, NULL );
                        // get requested NV index from msg
    if( nvIndex >= NUM_NVS ) {
        response.exp.code = NM_NV_fetch_fail;
        outLength = 1;
        nvName = "<invalid index>";
    } else {          // construct a response
        pNvValue = & NvValueTable[ nvIndex ];
        outLength = pNvValue->size + 2;    // add code and index
        response.exp.code = NM_NV_fetch_succ;
        pResponse = ( Byte * )response.exp.data;
        if( nvIndex < NV_IDX_ESC )
            * pResponse++ = nvIndex;        // unescaped
        else {
            * pResponse++ = NV_IDX_ESC;            // escape code
            * pResponse++ = hiByte( nvIndex );   // high byte
            * pResponse++ = loByte( nvIndex );   // low byte
            outLength += 2;                    // index is 3 bytes, not 1
        }
        memcpy( pResponse, pNvValue->data, pNvValue->size );
        nvName = pNvValue->name;
    }

    niError = NiSendResponse( & response, outLength, IN_MSG_HDR.tag );

    if( reportFlag ) printf( "Received NV fetch network mgmt msg "
        "for %s\n", nvName );

    NiErrorDisplay( "Error sending response to NV fetch", niError );
    return reportFlag || verboseFlag || ( niError != NI_OK );
}

/*************************************************************
 *
 *          Handle incoming NV poll messages to this node
 *
 *************************************************************/

Bool NiHandleNvPoll( ExpAppBuffer * pMsgIn ) {
    Word          nvIndex;
    nv_struct   * pNvConfig;
    NvData      * pNvValue;
    const char  * nvName;
    Byte          outLength;
    MsgData       response;
    NICode        niError;
    Bool          authOK;

    // Find this NV in the NV config table
    nvIndex = matchNvConfig( & pMsgIn->data.unv );

    if( nvIndex != NULL_NV_INDEX ) {       // found the NV
        pNvConfig = & NvConfigTable[ nvIndex ];
                        // point to NV config table
        pNvValue = & NvValueTable[ nvIndex ];
                        // point to NV data table
        authOK = ( IN_MSG_HDR.auth || ! pNvConfig->auth );
        // Message is authentic if it was authenticated by the Neuron or
        // authentication is not configured for this NV
        nvName = pNvValue->name;
    } else nvName = "<invalid selector>";

    response.unv = pMsgIn->data.unv;     // send back the same selector
    response.unv.direction = ~ response.unv.direction;
        // reverse the direction bit to send it back

    if( nvIndex != NULL_NV_INDEX && authOK && onlineFlag ) {
                   // there was a match
        memcpy( response.unv.data, pNvValue->data, pNvValue->size );
                // fetch the value
        outLength = pNvValue->size + 2;      // including selector
        numPolls++;
    } else outLength = 2;       // no match or off-line, respond anyway

    niError = NiSendResponse( & response, outLength, IN_MSG_HDR.tag );
                // add the selector size
    if( reportFlag && nvIndex != NULL_NV_INDEX )
         printf( "Received NV poll for %s\n", nvName );

    NiErrorDisplay( "Error sending response to NV poll", niError );
    return reportFlag || verboseFlag || ( niError != NI_OK );
}

/*************************************************************
 *
 *          Handle incoming NV update messages to this node
 *
 *************************************************************/

Bool NiHandleNvUpdate( ExpAppBuffer * pMsgIn ) {
    return handleNetvarUpdate( & pMsgIn->data.unv,
        (Byte)(IN_MSG_HDR.length - 2), (Bool)IN_MSG_HDR.auth );
}

/*************************************************************
 *
 *          Handle incoming application messages to this node
 *
 *************************************************************/

Bool NiHandleAppMsgs( ExpAppBuffer * pMsgIn ) {
    Byte        response;
    NICode      niError = NI_OK;
    Byte        inLength;
    Byte        code;

    numMessages++;
    inLength = IN_MSG_HDR.length - 1;    // subtract code
    dataLength += inLength;              // not counting code
    code = pMsgIn->data.exp.code;

        // if node is off-line and a request message is received
    if( IN_MSG_HDR.st == REQUEST && ! onlineFlag ) {
        response = ( code >= NA_foreign_msg ) ?
             NA_foreign_offline : NA_appl_offline;

        // return off-line response
        niError = NiSendResponse( ( MsgData * ) & response, 1,
            IN_MSG_HDR.tag );
        NiErrorDisplay( "Error sending response to explicit message", niError );
    }
    if( reportFlag ) {
        printf( "Received explicit application message\n" );
        printf( "Service type %s\n", svcTable[ IN_MSG_HDR.st ] );
        printf( "Message code = 0x%x\n", code );
        printf( "Authentication %s\n", IN_MSG_HDR.auth ? "on" : "off" );
        printf( "Length = %d\n", inLength );
    }
    return reportFlag || verboseFlag || ( niError != NI_OK );
}

/*************************************************************
 *
 *          Handle incoming wink messages to this node
 *
 *************************************************************/

Bool NiHandleWink( ExpAppBuffer * pMsgIn ) {
    if( reportFlag ) {
        printf( "Received Wink network mgmt msg\n" );
        printf( "Service type %s\n", svcTable[ IN_MSG_HDR.st ] );
        printf( "Length = %d\n", IN_MSG_HDR.length - 1 );
    }
    printf( "\a" );      // Ding!
    return reportFlag || verboseFlag;
}

/*************************************************************
 *
 *          Handle incoming service pin messages to this node
 *
 *************************************************************/

Bool NiHandleServicePinMsg( ExpAppBuffer * pMsgIn ) {
    NM_service_pin_msg * servPinMsg;
    Byte               * applID;

    servPinMsg = ( NM_service_pin_msg * ) & pMsgIn->data.exp;

    if( reportFlag ) {
        printf( "Received service pin message\n" );
        printf( "Neuron ID =" );
        printArray( servPinMsg->neuron_id, NEURON_ID_LEN, " %02X" );

        printf( "; Appl ID " );
        applID = servPinMsg->id_string;

       // print in hex for LonMark application ID, otherwise ASCII
        if( * applID & 0x80 )
             printArray( applID, ID_STR_LEN, ":%02X" );
        else printArray( applID, ID_STR_LEN, "%c" );
        printf( "\n" );
    }
    return reportFlag || verboseFlag;
}

/*************************************************************
 *
 *          Handle uplink network interface commands
 *
 *************************************************************/

Bool NiHandleUplinkCmd( ExpAppBuffer * pMsgIn ) {
        // something other than an incoming message

    if( reportFlag ) {
        if( pMsgIn->ni_hdr.noq.cmd == niRESET )
            printf( "Received uplink local reset\n" );
        else
            printf( "Received uplink command 0x%x\n", pMsgIn->ni_hdr.noq.cmd );
    }
    return reportFlag || verboseFlag;
}

/*************************************************************
 *
 *          Handle uplink out-of-order responses
 *
 *************************************************************/

Bool NiHandleInvalidResp ( ExpAppBuffer * pMsgIn ) {
    if( reportFlag ) {
        printf( "Received out-of-order response from network interface\n" );
        printf( "Message tag = %d\n", IN_MSG_HDR.tag );
    }
    return reportFlag || verboseFlag;
}

/*************************************************************
 *
 *            Command to report incoming traffic statistics
 *
 *************************************************************/

void traffic( void ) {

    printf( "Appl msgs received \t= %u\n", numMessages );
    printf( "Msg data received \t= %lu bytes\n", dataLength );
    printf( "NV updates received \t= %u\n", numUpdates );
    printf( "NV polls received \t= %u\n", numPolls );
    printf( "Clearing message and NV accumulators\n" );

    dataLength = numMessages = numPolls = numUpdates = 0;
}

/*
 ****************************************************************************
 *
 *      Store NV config table on disk
 *
 ****************************************************************************
 */

void storeNvConfig( void ) {
    int fd;
    unsigned byteCount;

    fd = open( "NVCONFIG.TBL", O_WRONLY | O_CREAT | O_TRUNC | O_BINARY,
        S_IREAD | S_IWRITE );
    if( fd < 0 ) {
        printf( "Unable to open network variable config file\n" );
        return;
    }
    byteCount = write( fd, NvConfigTable, sizeof( NvConfigTable ) );
    if( byteCount != sizeof( NvConfigTable ) )
        printf( "Unable to write to network variable config file\n" );
    else printf( "Network variable config table stored\n" );

    close( fd );
}

/*************************************************************
 *
 *          Load NV config table from disk
 *
 *************************************************************/

void loadNvConfig( void ) {       // called from main() startup code
    int fd;
    unsigned byteCount;

    fd = open( "NVCONFIG.TBL", O_RDONLY | O_BINARY );
    if( fd < 0 ) return;        // No file yet

    byteCount = read( fd, NvConfigTable, sizeof( NvConfigTable ) );
    if( byteCount != sizeof( NvConfigTable ) )
        printf( "Unable to load NV config table from file\n" );
    else printf( "Network variable config table loaded\n" );

    close( fd );
}

/*************************************************************
 *
 *    Handle an incoming network variable update message
 *
 *************************************************************/

Bool handleNetvarUpdate( UnprocessedNV * inNV, Byte inLength, Bool inAuth ) {
    Word        nvIndex;
    nv_struct * pNvConfig;

    // Find this NV in the NV config table
    nvIndex = matchNvConfig( inNV );
    if( nvIndex == NULL_NV_INDEX ) return FALSE;   // not found

    pNvConfig = & NvConfigTable[ nvIndex ];  // point to NV config table
    if( ! inAuth && pNvConfig->auth ) return FALSE;
    // NV is configured for authentication, but update was not authenticated

    return updateNvValue( nvIndex, inNV->data, inLength );
}

/*************************************************************
 *
 *    Routine to update the value of a network variable
 *
 *************************************************************/

Bool updateNvValue( Word nvIndex, Byte * data, int size ) {
    NvData  * pNvValue;

    pNvValue = & NvValueTable[ nvIndex ];  // point to NV data table

    if( size != pNvValue->size ) return FALSE;
            // Reject update if wrong size

    memcpy( pNvValue->data, data, size );   // update the NV value

    numUpdates++;
    if( reportFlag ) {
        printf( "Updated %s with : ", pNvValue->name );
        pNvValue->printFunc( pNvValue->data );
    }
    return reportFlag || verboseFlag;
}

/*************************************************************
 *
 *      matchNvConfig - find an entry in NV config table
 *      returns index of entry, or -1 if not found
 *
 *************************************************************/

Word matchNvConfig( UnprocessedNV * pInNv ) {

    nv_struct     * pNvConfig = NvConfigTable;
    Word            nvIndex;

    for( nvIndex = 0; nvIndex < NUM_NVS; nvIndex++, pNvConfig++ ) {
        if( pInNv->NV_selector_lo == pNvConfig->selector_lo
        &&  pInNv->NV_selector_hi == pNvConfig->selector_hi
        &&  pInNv->direction == pNvConfig->direction )
            return nvIndex;
    }
    return NULL_NV_INDEX;          // not found - return error
}

/*************************************************************
 *
 *      Utility function to return a possibly escaped
 *      NV index from a message.  Used by query/update NV config
 *      and NV fetch network management messages
 *
 *************************************************************/

static Word getNvIndex( ExpAppBuffer * pMsgIn,
    nv_struct ** pNextByte ) {

    Word nvIndex;

    Byte * pMsgData = pMsgIn->data.exp.data;
            // point to first data byte of message

    nvIndex = * pMsgData ++;  // get first byte of message

    if( nvIndex == NV_IDX_ESC ) {        // this is an escape sequence
                        // convert Neuron long to Intel word
        nvIndex = makeWord( pMsgData[ 0 ], pMsgData[ 1 ] );
        pMsgData += 2;
    }
    if( pNextByte ) * pNextByte = ( nv_struct * ) pMsgData;
            // return pointer to byte after the index
    return nvIndex;
}
