/* HAUIF.C -- Example LONWORKS(R) host application user interface.
 * Copyright (c) 1993 by Echelon Corporation.
 * All Rights Reserved.
 *
 * Date last modified: 11/17/94
 */

#include <ctype.h>                  /* For toupper()                        */
#include <stdlib.h>                 /* For exit()                           */
#include <string.h>                 /* For strlen(), strncpy()              */
#include <stdio.h>                  /* For printf()                         */
#include <conio.h>                  /* for kbhit( ) and getch( )            */
#include "ni_msg.h"
#include "ni_mgmt.h"
#include "hauif.h"

#define BUF_SIZE 80

const char  * svcTable[ 4 ]
    = { "Ackd", "Repeatd", "Unackd", "Request" };

/*************************** Function Prototypes ****************************/

/*  Command interpreter function handlers */

void    helpCmd( void );       // in HAUIF.C
void    nullCmd( void ) { }

extern  void    verbose( void );        // in HA.C
extern  void    traffic( void );        // in APPLMSG.C
extern  void    NV_table( void );       // in APPLMSG.C
extern  void    NV_update( void );      // in APPLMSG.C
extern  void    NV_poll( void );        // in APPLMSG.C

/*
 ****************************************************************************
 * Command table structure
 ****************************************************************************
 */

typedef struct  {
    char    letter;                 /* command letter */
    void    ( * func )( void );     /* handler function */
    char    * help_text;
} command_struct;

const static command_struct command_table[ ] = {

{     'E', nullCmd,    "(E)xit this application and return to DOS"       },
{     'H', helpCmd,    "(H)elp with commands",                           },
{     'N', NV_table,    "(N)etwork Variable configuration table",         },
{     'P', NV_poll,     "(P)oll input network variable"                   },
{     'T', traffic,     "Incoming network (T)raffic summary"              },
{     'U', NV_update,   "(U)pdate network variable"                       },
{     'V', verbose,     "Control (V)erbose modes"                         },
{     '\r', nullCmd,   ""                                                },
{     '\0' }
};

/*
 ****************************************************************************
 *
 * helpCmd().  Display help text
 *
 ****************************************************************************
 */

void helpCmd( void ) {
    const command_struct * pCmd;

    for( pCmd = command_table; pCmd->letter != '\r';  pCmd++ )
        printf( "%c -- %s.\n", pCmd->letter, pCmd->help_text );
}

/*
 ****************************************************************************
 *
 * processCmd().  Process a command from the keyboard
 *
 ****************************************************************************
 */

Bool processCmd( const char * prompt ) {       // return TRUE on exit
    char ch;
    const command_struct * pCmd;

    if( !kbhit( ) ) return FALSE;
    ch = getch( );        // read a character from keyboard
    ch = toupper( ch );

    for( pCmd = command_table; pCmd->letter; pCmd++ ) {
        if( pCmd->letter == ch ) {
            printf( "%s\n", pCmd->help_text );   // echo command
            if( ch == 'E' ) return TRUE;        // exit command
            ( pCmd->func ) ( );        // dispatch command
            printf( prompt );
            return FALSE;
       }
    }
        // List all the commands

    printf("\nDidn't recognize that command, valid commands are:\n" );
    helpCmd( );
    printf( prompt );
    return FALSE;
}

/*
 ****************************************************************************
 *
 *      getInteger - prompt and read an integer from the command line
 *
 ****************************************************************************
 */

int getInteger( const char * prompt, int default_val ) {
    char input_buf[ BUF_SIZE ];
    int number;

    for( ;; ) {
        printf( prompt );
        printf( " [%d] \t:", default_val );
        gets( input_buf );                  // read a line from keyboard
        if( !input_buf[ 0 ] ) return default_val;   // Empty line
        if( sscanf( input_buf, "%u", &number ) == 1 ) return number;
        printf( "Invalid integer\n" );
    }
}

/*
 ****************************************************************************
 *
 *      getChoice - prompt and read a binary choice from the command line
 *
 ****************************************************************************
 */

Bool getChoice( const char * prompt, char trueChoice, char falseChoice ) {
        // default choice is FALSE
    char ch;

    for( ;; ) {
        printf( prompt );
        ch = getch( );
        ch = toupper( ch );      // read a character from keyboard
        if( ch == '\r'|| ch == toupper( falseChoice ) ) {
            printf( "%c\n", falseChoice );
            return FALSE;
        }
        if( ch == toupper( trueChoice ) ) {
            printf( "%c\n", trueChoice );
            return TRUE;
        }
        printf( "Invalid choice\n" );
    }
}

/*
 ****************************************************************************
 *
 *      printArray - print an array of bytes to stdout and the output file
 *
 ****************************************************************************
 */

void printArray( const void * data, int length, const char * format ) {
    Byte * pByte;

    for( pByte = ( Byte * ) data; length--; pByte++ )
        printf( format, * pByte );
}

/*
 ****************************************************************************
 *
 * errorDisplay().  Display an error message. Split across two lines
 * if necessary
 *
 ****************************************************************************
 */

void errorDisplay( const char * str, const char * err ) {
    if( strlen( str ) + strlen( err ) < 78 )
         printf( "%s: %s\n", str, err );
    else printf( "%s:\n    %s\n", str, err );
        /* Message too long for one line, split on two lines. */
}
