/* NI_MGMT.H -- Host application network management codes and messages.
 * Copyright (c) 1992, 1993 by Echelon Corporation.
 * All Rights Reserved.
 *
 * Date last modified: 11/17/94
 *
 * Network management codes and messages for an application node.
 * This file contains a subset of the LonTalk network management structures.
 * The types have been changed where necessary because of the difference
 * in representations between Neuron and MS-DOS objects.
 */

#include "ha_comn.h"

#define ID_STR_LEN 8            // program ID length

#define NULL_IDX  15            // unused address table index

typedef struct {
    Bits    selector_hi : 6;
    Bits    direction   : 1;    // use NvDirection
    Bits    priority    : 1;
    Byte    selector_lo;
    Bits    addr_index  : 4;
    Bits    auth        : 1;
    Bits    service     : 2;   // use ServiceType
    Bits    turnaround  : 1;
} nv_struct;

typedef enum { NV_IN = 0, NV_OUT = 1 } NvDirection;

#define NV_IDX_ESC               0xFF       // escape code for NV index

#define NM_update_nv_cnfg        0x6B
#define NM_update_nv_cnfg_fail   0x0B
#define NM_update_nv_cnfg_succ   0x2B

typedef struct {
    Byte        code;
    nv_struct   nv_cnfg;
} NM_query_nv_cnfg_response;

#define NM_query_nv_cnfg        0x68
#define NM_query_nv_cnfg_fail   0x08
#define NM_query_nv_cnfg_succ   0x28

typedef enum {
    APPL_OFFLINE  = 0,
    APPL_ONLINE   = 1,
    APPL_RESET    = 2,
    CHANGE_STATE  = 3
} nm_node_mode;

#define NM_set_node_mode 0x6C

typedef enum {
    ABSOLUTE           = 0,
    READ_ONLY_RELATIVE = 1,
    CONFIG_RELATIVE    = 2,
} nm_mem_mode;

typedef enum {
    NO_ACTION      = 0,
    BOTH_CS_RECALC = 1,
    CNFG_CS_RECALC = 4,
    ONLY_RESET     = 8,
    BOTH_CS_RECALC_RESET = 9,
    CNFG_CS_RECALC_RESET = 12,
} nm_mem_form;

typedef struct {
    Byte    code;
    Byte    mode;
    Byte    offset_hi;
    Byte    offset_lo;
    Byte    count;
    Byte    form;                    // followed by the data
} NM_write_memory_request;

#define NM_write_memory 0x6E

typedef struct {
    Byte    length_hi;
    Byte    length_lo;
    Byte    num_netvars;
    Byte    version;         // version 0 format
    Byte    mtag_count;
} snvt_struct_v0;

typedef struct {
    Byte    length_hi;
    Byte    length_lo;
    Byte    num_netvars_lo;
    Byte    version;          // version 1 format
    Byte    num_netvars_hi;
    Byte    mtag_count;
} snvt_struct_v1;

// Partial list of SNVT type index values

typedef enum {
    SNVT_str_asc  = 36,
    SNVT_lev_cont = 21,
    SNVT_lev_disc = 22,
    SNVT_count_f  = 51,
} SNVT_t;

typedef struct {
    Bits    nv_config_class     :1;
    Bits    nv_auth_config      :1;
    Bits    nv_priority_config  :1;
    Bits    nv_service_config   :1;
    Bits    nv_offline          :1;
    Bits    nv_polled           :1;
    Bits    nv_sync             :1;
    Bits    ext_rec             :1;
    Byte    snvt_type_index;            // use enum SNVT_t
} snvt_desc_struct;

typedef struct {
    Bits    unused :3;
    Bits    nc     :1;     // array count
    Bits    sd     :1;     // self-documentation
    Bits    nm     :1;     // network variable name
    Bits    re     :1;     // rate estimate
    Bits    mre    :1;     // max rate estimate
} snvt_ext_rec_mask;

// Network management message codes

#define NM_query_SNVT           0x72
#define NM_query_SNVT_fail      0x12
#define NM_query_SNVT_succ      0x32

typedef struct {
    Byte    code;
    Byte    offset_hi;     // big-endian 16-bits
    Byte    offset_lo;
    Byte    count;
} NM_query_SNVT_request;

#define NM_wink 0x70

#define NM_NV_fetch         0x73
#define NM_NV_fetch_fail    0x13
#define NM_NV_fetch_succ    0x33

// Explicit application message codes

#define NA_appl_msg         0x00
#define NA_appl_offline     0x3F
#define NA_foreign_msg      0x40
#define NA_foreign_offline  0x4F

#define NA_last_msg_code    0x4F

typedef struct {
    Byte        code;
    Byte        neuron_id[ NEURON_ID_LEN ];
    Byte        id_string[ ID_STR_LEN ];
} NM_service_pin_msg;

#define NM_service_pin      0x7F
